@@ -197,6 +197,13 @@ while(<$fh>){
   }
   if($opts{""}){
     ############################# noun ######################################
+    # Shortcuts
+    if($opts{"_"}){
+    	$opts{""}=1; $opts{"__"}=1;
+    }
+    if($opts{"_"}){
+    	$opts{"__"}=1; $opts{"__"}=1;
+    }
     # note that the noun may have several plural forms (see, for example,
     # ). When one of the plural forms isn't explicitly specified, wolig
     # tries to guess, based on simplistic heuristics that work for the majority
@@ -336,6 +343,7 @@ while(<$fh>){
 	# different from the ah below...
         #outword $smichut."eha" unless $no_ah;
       }
+      unless ($opts{"__"}){
       outword $smichut."",   ",,/"; # possessives (kinu'im)
       outword $smichut."e", ",,/";
       outword $smichut."",   ",,/";
@@ -346,6 +354,7 @@ while(<$fh>){
       outword $smichut."ah",  ",,/";
       outword $smichut."a",  ",,/";
       outword $smichut."a",  ",,/";
+      }
     }
     # related plural noun forms
     # note: don't combine the $plural_.. ifs, nor use elsif, because some
@@ -372,6 +381,7 @@ while(<$fh>){
       # (We write patach followed by a consonant yud as "y", and later this will
       # give us the chance to automatically double it as necessary by the
       # Academia's ktiv male rules)
+      unless ($opts{"__"}){
       outword $smichut."y",        ",,/"; # possessives (kinu'im)
       outword $smichut."",      ",,/";
       outword $smichut."",       ",,/";
@@ -382,6 +392,7 @@ while(<$fh>){
       outword $smichut."",       ",,/";
       outword $smichut_orig."", ",,/";
       outword $smichut_orig."", ",,/";
+      }
     }
     if($plural_iim){
       # I currently decided that in Hebrew, unlike Arabic, only specific
@@ -398,6 +409,7 @@ while(<$fh>){
       $smichut=$xword;
       my $smichut_orig=$xword_orig;
       outword $smichut_orig."-", ",,"; # smichut
+      unless ($opts{"__"}){
       outword $smichut."y",        ",,/"; # possessives (kinu'im)
       outword $smichut."",      ",,/";
       outword $smichut."",       ",,/";
@@ -408,6 +420,7 @@ while(<$fh>){
       outword $smichut."",       ",,/";
       outword $smichut_orig."", ",,/";
       outword $smichut_orig."", ",,/";
+      }
     }
     if($plural_ot){
       my $xword=$word;
@@ -434,6 +447,7 @@ while(<$fh>){
       
       $smichut=$xword."";
       outword $smichut."-",   ",,"; # smichut
+      unless ($opts{"__"}){
       outword $smichut."y",   ",,/"; # possessives (kinu'im)
       outword $smichut."", ",,/";
       outword $smichut."",  ",,/";
@@ -444,6 +458,7 @@ while(<$fh>){
       outword $smichut."",  ",,/";
       outword $smichut."", ",,/";
       outword $smichut."", ",,/";
+      }
     }
     if($plural_iot){
       my $xword=$word;
@@ -457,6 +472,7 @@ while(<$fh>){
       outword $xword."",   ",";
       $smichut=$xword."";
       outword $smichut."-",   ",,"; # smichut
+      unless ($opts{"__"}){
       outword $smichut."y",   ",,/"; # possessives (kinu'im)
       outword $smichut."", ",,/";
       outword $smichut."",  ",,/";
@@ -467,6 +483,7 @@ while(<$fh>){
       outword $smichut."",  ",,/";
       outword $smichut."", ",,/";
       outword $smichut."", ",,/";
+      }
     }
     if($plural_xot){
       my $xword=$word;
@@ -480,6 +497,7 @@ while(<$fh>){
       outword $xword."",   ",";
       $smichut=$xword."";
       outword $smichut."-",   ",,"; # smichut
+      unless ($opts{"__"}){
       outword $smichut."y",   ",,/"; # possessives (kinu'im)
       outword $smichut."", ",,/";
       outword $smichut."",  ",,/";
@@ -490,6 +508,7 @@ while(<$fh>){
       outword $smichut."",  ",,/";
       outword $smichut."", ",,/";
       outword $smichut."", ",,/";
+      }
     }
     if($plural_bizarre){
       # User specified plural for bizarre cases; For example, the plural of
@@ -523,6 +542,7 @@ while(<$fh>){
       } else {
         die "Plural given for $word is of unrecognized form: $plural.";
       }
+      unless ($opts{"__"}){
       outword $smichut."y",        ",,/"; # possessives (kinu'im)
       outword $smichut."",      ",,/";
       outword $smichut."",       ",,/";
@@ -533,6 +553,7 @@ while(<$fh>){
       outword $smichut."",       ",,/";
       outword $smichut_orig."", ",,/";
       outword $smichut_orig."", ",,/";
+      }
     }
   } elsif($opts{""}){
     ############################# adjective ##################################
@@ -609,8 +630,10 @@ while(<$fh>){
     }
     my $nekeva_implicit = !($opts{"_"} || $opts{"_"} ||
     			    $opts{""});
+    # by checking for final i, we're basically checking for final  except
+    # in final  (see comment above on where we added the i)
     my $nekeva_t = $opts{"_"} ||
-    		   ($nekeva_implicit && substr($xword,-1,1) eq "");
+    		   ($nekeva_implicit && substr($xword,-2,2) eq "i");
     my $nekeva_h = $opts{"_"} ||
     		   ($nekeva_implicit && !$nekeva_t);
     if(exists($opts{""})){
